"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const http_1 = require("@reactive-js/http");
const observable_1 = require("@reactive-js/observable");
const capabilities_1 = require("./capabilities");
const httpResponseBody_1 = require("./httpResponseBody");
const option_1 = require("@reactive-js/option");
const pipe_1 = require("@reactive-js/pipe");
exports.sendHttpRequestUsingFetch = request => {
    const { cache, credentials, integrity, method, mode, redirect, referrerPolicy, uri, } = request;
    const url = uri.toString();
    const headers = http_1.httpRequestToUntypedHeaders(request);
    return observable_1.createObservable((subscriber) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        const abortController = new AbortController();
        subscriber.add(() => abortController.abort());
        subscriber.dispatch({
            type: 1,
        });
        try {
            const fetchResponse = yield fetch(url, {
                cache,
                credentials,
                headers,
                integrity,
                method,
                mode,
                redirect,
                referrerPolicy,
                signal: abortController.signal,
            });
            const responseHeaders = {};
            fetchResponse.headers.forEach((v, k) => {
                responseHeaders[k] = v;
            });
            const responseNoBody = http_1.parseHttpResponseFromHeaders(fetchResponse.status, responseHeaders, option_1.none);
            const loadBodyContent = () => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
                const { contentInfo } = responseNoBody;
                if (option_1.isSome(contentInfo)) {
                    const { contentLength, contentType: { type, subtype, params }, } = contentInfo;
                    const hasCharset = option_1.isSome(params["charset"]);
                    const responseIsText = hasCharset ||
                        type === "text" ||
                        subtype.indexOf("json") >= 0 ||
                        subtype.indexOf("text") >= 0 ||
                        subtype.indexOf("xml") >= 0;
                    if (responseIsText) {
                        return fetchResponse.text();
                    }
                    else if (contentLength > 0 && capabilities_1.supportsArrayBuffer) {
                        return fetchResponse.arrayBuffer();
                    }
                    else if (capabilities_1.supportsBlob) {
                        return fetchResponse.blob();
                    }
                }
                return yield "";
            });
            const bodyObservable = pipe_1.pipe(observable_1.fromPromise(loadBodyContent), observable_1.publish(subscriber, 1));
            const body = new httpResponseBody_1.HttpResponseBodyImpl(bodyObservable);
            subscriber.add(body);
            body.add(subscriber);
            const response = Object.assign(Object.assign({}, responseNoBody), { body });
            subscriber.dispatch({
                type: 4,
                response,
            });
        }
        catch (cause) {
            subscriber.dispose({ cause });
        }
    }));
};
//# sourceMappingURL=fetch.js.map